part of aargul_crud;

void _setDefaultAargulCrudFields() {
  AargulCrudManager.register(
    AargulCrudFieldType.submit.name,
    (fieldName, field) {
      field.hasInput = false;
      ValueNotifier<bool> enabled = ValueNotifier(!field.disabled);
      field.on(_fieldUpdatedEvent, (data) {
        enabled.value = !field.disabled;
        field._value = enabled.value;
      });
      return ValueListenableBuilder(
        valueListenable: enabled,
        builder: (context, value, child) => FilledButton(
          onPressed: value
              ? () {
                  field.disabled = true;
                  field.emit("submit");
                  field.on("submited", (v) {
                    field.disabled = false;
                  });
                }
              : null,
          focusable: value,
          child: SizedBox(
            height: 24,
            child: !value
                ? const SizedBox(
                    height: 24,
                    child: ProgressBar(),
                  )
                : Text(
                    field.label ?? (fieldName.toUpperCase()),
                    style: const TextStyle(fontSize: 20),
                  ),
          ),
        ),
      );
    },
    cellBuilder: (fieldName, field, values) => null,
  );
  AargulCrudManager.register(
    AargulCrudFieldType.input.name,
    (fieldName, field) {
      var controller = TextEditingController(text: field.value?.toString());
      bool self = false;
      field.addListener(() {
        if (!self) {
          controller.text = field.value ?? "";
        }
        self = false;
      });
      controller.addListener(() {
        if (controller.text != field.value || field.value == null) {
          self = true;
          field.value = controller.text;
        }
      });
      return InfoLabel(
        label: "${field.label ?? (fieldName.toUpperCase())}:",
        child: TextBox(
          controller: controller,
          enabled: !field.disabled,
          placeholder: field.placeholder,
          expands: false,
          maxLines: 1,
          minLines: 1,
          maxLength: 250,
        ),
      );
    },
  );
  AargulCrudManager.register(
    AargulCrudFieldType.password.name,
    (fieldName, field) {
      var controller = TextEditingController(text: field.value?.toString());

      bool self = false;
      field.addListener(() {
        if (!self) {
          controller.text = field.value ?? "";
        }
        self = false;
      });
      controller.addListener(() {
        if (controller.text != field.value || field.value == null) {
          self = true;
          field.value = controller.text;
        }
      });
      return InfoLabel(
        label: "${field.label ?? (fieldName.toUpperCase())}:",
        child: PasswordBox(
          controller: controller,
          enabled: !field.disabled,
          placeholder: field.placeholder,
          revealMode: PasswordRevealMode.peekAlways,
          obscuringCharacter: "*",
        ),
      );
    },
  );
  AargulCrudManager.register(
    AargulCrudFieldType.text.name,
    (fieldName, field) {
      var controller = TextEditingController(text: field.value?.toString());

      bool self = false;
      field.addListener(() {
        if (!self) {
          controller.text = field.value ?? "";
        }
        self = false;
      });
      controller.addListener(() {
        if (controller.text != field.value || field.value == null) {
          self = true;
          field.value = controller.text;
        }
      });
      return InfoLabel(
        label: "${field.label ?? (fieldName.toUpperCase())}:",
        child: TextBox(
          controller: controller,
          enabled: !field.disabled,
          placeholder: field.placeholder,
          expands: false,
          maxLines: null,
          minLines: 5,
        ),
      );
    },
  );
  AargulCrudManager.register(
    AargulCrudFieldType.phone.name,
    (fieldName, field) {
      var controller = TextEditingController(text: field.value?.toString());

      bool self = false;
      field.addListener(() {
        if (!self) {
          controller.text = field.value ?? "";
        }
        self = false;
      });
      controller.addListener(() {
        if (controller.text != field.value || field.value == null) {
          self = true;
          field.value = controller.text;
        }
      });
      return InfoLabel(
        label: "${field.label ?? (fieldName.toUpperCase())}:",
        child: TextBox(
          controller: controller,
          enabled: !field.disabled,
          placeholder: field.placeholder,
          expands: false,
          maxLines: 1,
          minLines: 1,
          inputFormatters: [FilteringTextInputFormatter.digitsOnly],
        ),
      );
    },
  );
  AargulCrudManager.register(
    AargulCrudFieldType.email.name,
    (fieldName, field) {
      var controller = TextEditingController(text: field.value?.toString());

      bool self = false;
      field.addListener(() {
        if (!self) {
          controller.text = field.value ?? "";
        }
        self = false;
      });
      controller.addListener(() {
        if (controller.text != field.value || field.value == null) {
          self = true;
          field.value = controller.text;
        }
      });
      return InfoLabel(
        label: "${field.label ?? (fieldName.toUpperCase())}:",
        child: TextBox(
          controller: controller,
          enabled: !field.disabled,
          placeholder: field.placeholder,
          expands: false,
          maxLines: 1,
          minLines: 1,
        ),
      );
    },
  );
  AargulCrudManager.register(
    AargulCrudFieldType.phone.name,
    (fieldName, field) {
      var controller = TextEditingController(text: field.value?.toString());

      bool self = false;
      field.addListener(() {
        if (!self) {
          controller.text = field.value ?? "";
        }
        self = false;
      });
      controller.addListener(() {
        if (controller.text != field.value || field.value == null) {
          self = true;
          field.value = controller.text;
        }
      });
      return InfoLabel(
        label: "${field.label ?? (fieldName.toUpperCase())}:",
        child: TextBox(
          controller: controller,
          enabled: !field.disabled,
          placeholder: field.placeholder,
          expands: false,
          maxLines: 1,
          minLines: 1,
          inputFormatters: [FilteringTextInputFormatter.digitsOnly],
        ),
      );
    },
  );
  AargulCrudManager.register(
    AargulCrudFieldType.number.name,
    (fieldName, field) {
      field.value = num.tryParse("${field.value}") ?? 0;
      return InfoLabel(
        label: "${field.label ?? (fieldName.toUpperCase())}:",
        child: ValueListenableBuilder(
          valueListenable: field,
          builder: (context, value, child) => NumberBox(
            onChanged: (value) => field.value = value,
            value: num.tryParse("$value") ?? 0,
            placeholder: field.placeholder,
            mode: SpinButtonPlacementMode.inline,
            inputFormatters: [FilteringTextInputFormatter.digitsOnly],
          ),
        ),
      );
    },
  );
  AargulCrudManager.register(
    AargulCrudFieldType.select.name,
    (fieldName, field) {
      // field.value = field.value != null && field.value is String ? field.value : null;
      return InfoLabel(
        label: "${field.label ?? (fieldName.toUpperCase())}:",
        child: ValueListenableBuilder(
            valueListenable: field,
            builder: (context, value, child) => ComboBox<dynamic>(
                  value: value,
                  items: (field.options as Map).entries.map((e) {
                    return ComboBoxItem<dynamic>(
                      value: e.key,
                      child: Text(e.value.toString()),
                    );
                  }).toList(),
                  onChanged: field.disabled ? null : (s) => field.value = s,
                )),
      );
    },
  );
  AargulCrudManager.register(
    AargulCrudFieldType.time.name,
    (fieldName, field) {
      field.value =
          field.value != null && field.value is DateTime ? field.value : null;
      return InfoLabel(
        label: "${field.label ?? (fieldName.toUpperCase())}:",
        child: ValueListenableBuilder(
            valueListenable: field,
            builder: (context, value, child) => TimePicker(
                  selected: value,
                  hourFormat: field.options["format"] != null
                      ? HourFormat.values.firstWhere(
                          (element) =>
                              "${field.options["format"]}" == element.name,
                          orElse: () => HourFormat.HH,
                        )
                      : HourFormat.HH,
                  onChanged: field.disabled ? null : (s) => field.value = s,
                )),
      );
    },
  );
  AargulCrudManager.register(
    AargulCrudFieldType.date.name,
    (fieldName, field) {
      field.value = field.value != null && field.value is DateTime
          ? field.value
          : null; // initialize data
      return InfoLabel(
        label: "${field.label ?? (fieldName.toUpperCase())}:",
        child: ValueListenableBuilder(
            valueListenable: field,
            builder: (context, value, child) => DatePicker(
                  selected: value,
                  showDay: field.options["day"] ?? true,
                  showMonth: field.options["month"] ?? true,
                  showYear: field.options["year"] ?? true,
                  onChanged: field.disabled ? null : (s) => field.value = s,
                )),
      );
    },
  );
}
